/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "bufferlist.h"

namespace bitGtk {

  BufferList::BufferList()
      : Gtk::EventBox(), m_buffer(NULL), m_table(NULL)
  {
    create_table();
  }


  BufferList::~BufferList()
  {}

  void BufferList::set_buffer( bit::RecordBuffer & buffer )
  {
    m_buffer = &buffer;
    buffer.signal_field_changed().connect(sigc::mem_fun(*this, &BufferList::on_field_value_changed));
    buffer.signal_record_changed().connect(sigc::mem_fun(*this, &BufferList::on_record_changed));
    setup_table();
  }

  void BufferList::setup_table( )
  {

    if (m_buffer == NULL)
      return;

    create_table();
    m_value_label.clear();

    m_value_label_left = 0;
    add_field_label( m_buffer->record() );
    add_field_value_label( m_buffer->record() );
    load_all_field_values();
  }

  unsigned int BufferList::add_field_label(bit::FieldBase::pointer field, unsigned int row, unsigned int depth) {
    std::ostringstream text;
    std::string name;
    Gtk::Label* label;
    Gtk::EventBox* eventbox;

    if (!m_table)
      return row;

    m_value_label_left = std::max(m_value_label_left, depth+1);

    if ( field->fields() > 0 ) {
      name = field->name();
      label = Gtk::manage(new Gtk::Label());
      text.str("");
      text << "<big><b>" << name << "</b></big>";
      label->set_justify(Gtk::JUSTIFY_LEFT);
      label->set_use_markup();
      label->set_markup(text.str());
      eventbox = Gtk::manage(new Gtk::EventBox());
      eventbox->add(*label);
      if (field->description().size() > 0 )
        m_tooltips.set_tip(*eventbox, field->description());
      m_table->attach(*eventbox, depth, depth+3, row, row+1);
      row++;
      for (bit::FieldBase::iterator iter = field->begin(); iter != field->end(); iter++)
        row = add_field_label( iter.pointer(), row, depth+1 );
    }
    else {
      name = field->name();
      label = Gtk::manage(new Gtk::Label());
      text.str("");
      text << "<big><b>" << name << "</b></big>";
      label->set_justify(Gtk::JUSTIFY_LEFT);
      label->set_use_markup();
      label->set_markup(text.str());
      eventbox = Gtk::manage(new Gtk::EventBox());
      eventbox->add(*label);
      if (field->description().size() > 0 )
        m_tooltips.set_tip(*eventbox, field->description());
      m_table->attach(*eventbox, depth, depth+1, row, row+1);
      return row+1;
    }
    return row;
  }

  unsigned int BufferList::add_field_value_label(bit::FieldBase::pointer field, unsigned int row) {
    std::ostringstream text;
    std::string name;
    Gtk::Label* label;
    Gtk::EventBox* eventbox;

    if (!m_table)
      return row;

    if ( field->fields() > 0 ) {
      row++;
      for (bit::FieldBase::iterator iter = field->begin(); iter != field->end(); iter++)
        row = add_field_value_label( iter.pointer(), row );
    }
    else {
      label = Gtk::manage(new Gtk::Label());
      label->set_justify(Gtk::JUSTIFY_LEFT);
      eventbox = Gtk::manage(new Gtk::EventBox());
      eventbox->add(*label);
      if (field->description().size() > 0 )
        m_tooltips.set_tip(*eventbox, field->description());
      m_table->attach(*eventbox, m_value_label_left, m_value_label_left+1, row, row+1);
      m_value_label[ field ] = label;
      return row+1;
    }
    return row;
  }

  void BufferList::on_record_changed() {
    load_all_field_values();
  }

  void BufferList::on_field_value_changed( bit::FieldBuffer fieldbuffer )
  {
    std::string text;
    text = std::string(" <tt>") + fieldbuffer.data().hex_string() + std::string("</tt> ");
    m_value_label[ fieldbuffer.field() ]->set_use_markup();
    m_value_label[ fieldbuffer.field() ]->set_markup(text);
  }

  void BufferList::set_field_value( bit::FieldBase::pointer field )
  {
    on_field_value_changed( bit::FieldBuffer( *m_buffer, field ) );
  }


  void BufferList::load_all_field_values()
  {
    if (! m_buffer )
      return;

    load_all_field_values( m_buffer->record() );
  }

  void BufferList::load_all_field_values(bit::FieldBase::pointer field)
  {
    if ( field->fields() > 0)
      for (bit::FieldBase::iterator iter = field->begin(); iter != field->end(); iter++)
        load_all_field_values( iter.pointer() );
    else
      set_field_value( field );
  }

  void BufferList::create_table( )
  {
    if (m_table != NULL)
      remove
        ();

    m_table = Gtk::manage(new Gtk::Table());
    m_table->set_homogeneous(false);
    m_table->set_spacings(3);
    add
      (*m_table);
  }

  void BufferList::enable_tooltips( bool b )
  {
    if (b)
      m_tooltips.enable();
    else
      m_tooltips.disable();
  }

}
